<?php

namespace Drupal\statistics_rolling;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\datetime\Plugin\Field\FieldType\DateTimeItemInterface;
use Drupal\statistics\StatisticsStorageInterface;
use Drupal\statistics\StatisticsViewsResult;

/**
 * Decorates the statistics service to count views over a rolling period.
 */
class RollingNodeStatisticsDatabaseStorage implements StatisticsStorageInterface {

  /**
   * @param \Drupal\statistics\StatisticsStorageInterface $originalService
   * @param \Drupal\Core\Database\Connection $connection
   * @param \Drupal\Component\Datetime\TimeInterface $time
   */
  public function __construct(
    private readonly StatisticsStorageInterface $originalService,
    protected Connection $connection,
    protected TimeInterface $time,
  ) {
  }

  /**
   * {@inheritdoc}
   * @throws \Exception
   * @see \Drupal\statistics\NodeStatisticsDatabaseStorage::recordView()
   * @SuppressWarnings(PHPMD.ShortVariable)
   */
  public function recordView($id): bool {
    $date = DrupalDateTime::createFromTimestamp($this->time->getRequestTime())
      ->format(DateTimeItemInterface::DATE_STORAGE_FORMAT);
    $this->connection
      ->merge('node_counter_rolling')
      ->key('nid', $id)
      ->key('date', $date)
      ->fields([
        'daycount' => 1,
        'date' => $date,
      ])
      ->expression('daycount', '[daycount] + 1')
      ->execute();

    return $this->originalService->recordView($id);
  }

  /**
   * {@inheritdoc}
   * @param int[] $ids
   */
  public function fetchViews($ids): array {
    return $this->originalService->fetchViews($ids);
  }

  /**
   * {@inheritdoc}
   * @SuppressWarnings(PHPMD.ShortVariable)
   */
  public function fetchView($id): StatisticsViewsResult|bool {
    return $this->originalService->fetchView($id);
  }

  /**
   * {@inheritdoc}
   * @return int[]
   */
  public function fetchAll($order = 'totalcount', $limit = 5): array {
    return $this->originalService->fetchAll($order, $limit);
  }

  /**
   * {@inheritdoc}
   * @SuppressWarnings(PHPMD.ShortVariable)
   */
  public function deleteViews($id): bool {
    return $this->originalService->deleteViews($id);
  }

  /**
   * {@inheritdoc}
   */
  public function resetDayCount(): void {
    $this->originalService->resetDayCount();
  }

  /**
   * {@inheritdoc}
   */
  public function maxTotalCount(): int {
    return $this->originalService->maxTotalCount();
  }

}
